@@ -20,6 +20,7 @@
  */
 /**
  * SECTION:gstpreset
+ * @title: GstPreset
  * @short_description: helper interface for element presets
  *
  * This interface offers methods to query and manipulate parameter preset sets.
@@ -58,6 +59,7 @@
 #endif
 
 #include "preset.h"
+#include "compat.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -81,6 +83,11 @@ static GQuark instance_list_quark = 0;
 /* max character per line */
 #define LINE_LEN 200
 
+//-- the iface
+
+G_DEFINE_INTERFACE (GstPreset, gst_preset, 0);
+
+
 static gboolean gst_preset_default_save_presets_file (GstPreset * self);
 
 #define preset_get_storage(self) \
@@ -205,11 +212,11 @@ preset_open_and_parse_header(GstPreset * self, const gchar *preset_path, gchar *
 {
   GKeyFile *in = g_key_file_new();
   GError *error = NULL;
-  
+
   if (g_key_file_load_from_file (in, preset_path, G_KEY_FILE_KEEP_COMMENTS|G_KEY_FILE_KEEP_TRANSLATIONS, &error)) {
     const gchar *element_name = G_OBJECT_TYPE_NAME (self);
     gchar *name = g_key_file_get_value (in, PRESET_HEADER, "element", NULL);
-    
+
     if (!name || strcmp (name, element_name)) {
       GST_WARNING ("Wrong element name in preset file %s", preset_path);
       goto Error;
@@ -254,7 +261,7 @@ preset_merge (GKeyFile *system, GKeyFile *user) {
   gchar *str;
   gchar **groups, **keys;
   gsize i, j, num_groups, num_keys;
-  
+
   /* copy file comment if there is any */
   if ((str = g_key_file_get_comment (user, NULL, NULL, NULL))) {
     g_key_file_set_comment (system, NULL, NULL, str, NULL);
@@ -276,7 +283,7 @@ preset_merge (GKeyFile *system, GKeyFile *user) {
     if (g_key_file_has_group (system, groups[i])) {
       g_key_file_remove_group (system, groups[i], NULL);
     }
-    
+
     keys = g_key_file_get_keys (user, groups[i], &num_keys, NULL);
     for (j = 0; j < num_keys; j++) {
       /* copy key comment if there is any */
@@ -314,7 +321,7 @@ gst_preset_default_get_preset_names (GstPreset * self)
     in_system = preset_open_and_parse_header (self, preset_system_path, &str_version_system);
     /* compare version to check for merge */
     if (in_system) {
-      if (!in_user || (in_user && (preset_parse_version (str_version_system) > 
+      if (!in_user || (in_user && (preset_parse_version (str_version_system) >
           preset_parse_version (str_version_user)))) {
         /* keep system presets */
         presets = in_system;
@@ -343,7 +350,7 @@ gst_preset_default_get_preset_names (GstPreset * self)
       gst_preset_default_save_presets_file (self);
     }
   }
-  
+
   if (presets) {
     GList *instances;
     gboolean found = FALSE;
@@ -351,7 +358,7 @@ gst_preset_default_get_preset_names (GstPreset * self)
     gchar **groups = g_key_file_get_groups (presets, &num_groups);
     gchar **preset_names = g_new (gchar*, num_groups);
     GSList *list=NULL,*node;
-    
+
     /* insert instance in instance list (if not yet there) */
     instances = (GList *) g_type_get_qdata (type, instance_list_quark);
     if (instances != NULL) {
@@ -445,7 +452,7 @@ gst_preset_default_load_preset (GstPreset * self, const gchar * name)
           if ((str = g_key_file_get_value (presets, name, property->name, NULL))) {
             GST_DEBUG ("setting value '%s' for property '%s'", str,
                 property->name);
-            
+
             g_value_init(&gvalue, property->value_type);
             if (gst_value_deserialize (&gvalue, str)) {
               g_object_set_property (G_OBJECT (self), property->name, &gvalue);
@@ -605,7 +612,7 @@ gst_preset_default_rename_preset (GstPreset * self, const gchar * old_name,
         g_key_file_set_comment (presets, new_name, NULL, str, NULL);
         g_free(str);
       }
-  
+
       keys = g_key_file_get_keys (presets, old_name, &num_keys, NULL);
       for (j = 0; j < num_keys; j++) {
         /* copy key comment if there is any */
@@ -618,9 +625,9 @@ gst_preset_default_rename_preset (GstPreset * self, const gchar * old_name,
         g_free(str);
       }
       g_strfreev(keys);
-  
+
       g_key_file_remove_group (presets, old_name, NULL);
-  
+
       return gst_preset_default_save_presets_file (self);
     } else {
       GST_WARNING ("no preset named %s", old_name);
@@ -681,13 +688,13 @@ gst_preset_default_get_meta (GstPreset * self, const gchar * name,
     const gchar * tag, gchar ** value)
 {
   GKeyFile *presets;
-  
+
   g_assert (value);
 
   /* get the presets from the type */
   if ((presets = preset_get_storage (self))) {
     gchar *key = g_strdup_printf ("_meta/%s",tag);
-    
+
     *value = g_key_file_get_value(presets, name, key, NULL);
     g_free (key);
     return TRUE;
@@ -863,8 +870,11 @@ gst_preset_get_meta (GstPreset * self, const gchar * name, const gchar * tag,
 /* class internals */
 
 static void
-gst_preset_class_init (GstPresetInterface * iface)
+gst_preset_default_init (GstPresetInterface * iface)
 {
+  GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT, "preset",
+      GST_DEBUG_FG_WHITE | GST_DEBUG_BG_BLACK, "preset interface");
+
   iface->get_preset_names = gst_preset_default_get_preset_names;
   iface->get_property_names = gst_preset_default_get_property_names;
 
@@ -875,66 +885,11 @@ gst_preset_class_init (GstPresetInterface * iface)
 
   iface->set_meta = gst_preset_default_set_meta;
   iface->get_meta = gst_preset_default_get_meta;
+
+  /* create quarks for use with g_type_{g,s}et_qdata() */
+  preset_quark = g_quark_from_static_string ("GstPreset::presets");
+  preset_user_path_quark = g_quark_from_static_string ("GstPreset::user_path");
+  preset_system_path_quark = g_quark_from_static_string ("GstPreset::system_path");
+  instance_list_quark = g_quark_from_static_string ("GstPreset::instances");
 }
 
-static void
-gst_preset_base_init (gpointer g_class)
-{
-  static gboolean initialized = FALSE;
-
-  if (!initialized) {
-    /* init default implementation */
-    GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT, "preset",
-        GST_DEBUG_FG_WHITE | GST_DEBUG_BG_BLACK, "preset interface");
-
-    /* create quarks for use with g_type_{g,s}et_qdata() */
-    preset_quark = g_quark_from_static_string ("GstPreset::presets");
-    preset_user_path_quark = g_quark_from_static_string ("GstPreset::user_path");
-    preset_system_path_quark = g_quark_from_static_string ("GstPreset::system_path");
-    instance_list_quark = g_quark_from_static_string ("GstPreset::instances");
-
-#if 0
-    property_list_quark = g_quark_from_static_string ("GstPreset::properties");
-
-    /* create interface properties, each element would need to override this
-     *   g_object_class_override_property(gobject_class, PROP_PRESET_NAME, "preset-name");
-     * and in _set_property() do
-     *   case PROP_PRESET_NAME: {
-     *     gchar *name = g_value_get_string (value);
-     *     if (name)
-     *       gst_preset_load_preset(self, name);
-     *   } break;
-     */
-    g_object_interface_install_property (g_class,
-      g_param_spec_string ("preset-name",
-      "preset-name property",
-      "load given preset",
-      NULL,
-      G_PARAM_WRITABLE));
-#endif
-
-    initialized = TRUE;
-  }
-}
-
-GType
-gst_preset_get_type (void)
-{
-  static GType type = 0;
-
-  if(G_UNLIKELY(!type)) {
-    const GTypeInfo info = {
-      sizeof (GstPresetInterface),
-      (GBaseInitFunc) gst_preset_base_init,     /* base_init */
-      NULL,                     /* base_finalize */
-      (GClassInitFunc) gst_preset_class_init,   /* class_init */
-      NULL,                     /* class_finalize */
-      NULL,                     /* class_data */
-      0,
-      0,                        /* n_preallocs */
-      NULL                      /* instance_init */
-    };
-    type = g_type_register_static (G_TYPE_INTERFACE, "GstPreset", &info, 0);
-  }
-  return type;
-}
