@@ -112,7 +112,7 @@ eof_error:
     gchar **preset_names = g_new (gchar*, g_list_length (klass->presets) + 1);
     GList *node;
     guint i=0;
-    
+
     for (node = klass->presets; node; node = g_list_next (node)) {
       preset_names[i++] = g_strdup(node->data);
     }
@@ -145,7 +145,7 @@ gboolean gstbml_preset_load_preset(GstObject *self, GstBML *bml, GstBMLClass *kl
 
       tracks=*data++;
       params=*data++;
-      
+
       if(klass->voice_type) {
         voice_class=G_OBJECT_CLASS(g_type_class_ref(klass->voice_type));
       }
@@ -493,27 +493,37 @@ void gstbml_preset_finalize(GstBMLClass *klass) {
 
 //-- common class functions
 
-/*
+/**
  * gstbml_convert_names:
+ * @klass: the instance class
+ * @tmp_name: name to build @name and @nick from
+ * @tmp_desc: desc to build @desc from
+ * @name: target for name
+ * @nick: target for nick
+ * @desc: target for description
  *
  * Convert charset encoding and make property-names unique.
  */
-void gstbml_convert_names(GObjectClass *klass, gchar *tmp_name, gchar *tmp_desc, gchar **_name, gchar **nick, gchar **desc) {
-  gchar *name,*ptr1,*ptr2;
+void gstbml_convert_names(GObjectClass *klass, gchar *tmp_name, gchar *tmp_desc, gchar **name, gchar **nick, gchar **desc) {
+  gchar *cname,*ptr1,*ptr2;
 
   GST_DEBUG("        tmp_name='%s'",tmp_name);
-  // @todo: do we want different charsets for BML_WRAPPED/BML_NATIVE? 
-  name=g_convert(tmp_name,-1,"ASCII","WINDOWS-1252",NULL,NULL,NULL);
+  // @todo: do we want different charsets for BML_WRAPPED/BML_NATIVE?
+  cname=g_convert(tmp_name,-1,"ASCII","WINDOWS-1252",NULL,NULL,NULL);
+  if(!cname) {
+    // weak fallback when conversion failed
+    cname=g_strdup(tmp_name);
+  }
   if(nick) {
     *nick=g_convert(tmp_name,-1,"UTF-8","WINDOWS-1252",NULL,NULL,NULL);
   }
   if(desc && tmp_desc) {
     *desc=g_convert(tmp_desc,-1,"UTF-8","WINDOWS-1252",NULL,NULL,NULL);
   }
-  g_strcanon(name, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "-_", '-');
+  g_strcanon(cname, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "-_", '-');
 
   // remove leading '-'
-  ptr1=ptr2=name;
+  ptr1=ptr2=cname;
   while(*ptr2=='-') ptr2++;
   // remove double '-'
   while(*ptr2) {
@@ -530,38 +540,53 @@ void gstbml_convert_names(GObjectClass *klass, gchar *tmp_name, gchar *tmp_desc,
   while(*ptr1=='-') *ptr1++='\0';
 
   // name must begin with a char
-  if(!g_ascii_isalpha(name[0])) {
-    gchar *old_name=name;
-    name=g_strconcat("Par_",old_name,NULL);
+  if(!g_ascii_isalpha(cname[0])) {
+    gchar *old_name=cname;
+    cname=g_strconcat("Par_",old_name,NULL);
     g_free(old_name);
   }
 
   // check for already existing property names
-  if(g_object_class_find_property(klass, name)) {
-    gchar *old_name=name;
+  if(g_object_class_find_property(klass, cname)) {
+    gchar *old_name=cname;
     gchar postfix[5];
     gint i=0;
 
     // make name uniqe
-    name=NULL;
+    cname=NULL;
     do {
-      if(name) g_free(name);
+      if(cname) g_free(cname);
       snprintf(postfix,5,"_%03d",i++);
-      name=g_strconcat(old_name,postfix,NULL);
-    } while(g_object_class_find_property(klass, name));
+      cname=g_strconcat(old_name,postfix,NULL);
+    } while(g_object_class_find_property(klass, cname));
      g_free(old_name);
   }
-  *_name=name;
+  *name=cname;
 }
 
-/*
+/**
  * gstbml_register_param:
+ * @klass: the instance class
+ * @prop_id: the property number
+ * @type: the parameter type
+ * @enum_type: the enum type (or 0)
+ * @name: the property name
+ * @nick: the property nick
+ * @desc: the property description
+ * @flags: extra property flags
+ * @min_val: minimum value
+ * @max_val: maximum value
+ * @no_val: unset value (neutral)
+ * @def_val: default value
  *
- * normalize data and register property
+ * Normalize data and create a paramspec.
+ *
+ * Returns: the param spec
  */
-GParamSpec *gstbml_register_param(GObjectClass *klass,gint prop_id, gint type, GType enum_type, gchar *name, gchar *nick, gchar *desc, gint flags, gint min_val, gint max_val, gint no_val, gint def_val) {
+GParamSpec *gstbml_register_param(GObjectClass *klass,gint prop_id, GstBMLParameterTypes type, GType enum_type, gchar *name, gchar *nick, gchar *desc, gint flags, gint min_val, gint max_val, gint no_val, gint def_val) {
   GParamSpec *paramspec = NULL;
   gint saved_min_val=min_val,saved_max_val=max_val,saved_def_val=def_val;
+  GParamFlags pspec_flags=G_PARAM_WRITABLE|GST_PARAM_CONTROLLABLE;
 
   GST_DEBUG("        name='%s', nick='%s', type=%d, flags=0x%x : val [%d ... d=%d/n=%d ... %d]",name,nick,type,flags, min_val,def_val,no_val,max_val);
   //*def = defval;
@@ -588,15 +613,15 @@ GParamSpec *gstbml_register_param(GObjectClass *klass,gint prop_id, gint type, G
     }
     def_val=no_val;
   }
+  else {
+    pspec_flags|=G_PARAM_READABLE;
+  }
   GST_DEBUG("        val [%d ... d=%d ... %d]",min_val,def_val,max_val);
 
   switch(type) {
     case PT_NOTE:
-      paramspec=g_param_spec_string(name, nick, desc,
-          NULL, G_PARAM_READWRITE|GST_PARAM_CONTROLLABLE);
-      /* @todo: what about using an enum type (define in gst-buzztard) here to
-       * be able to detect this type in UIs
-       */
+      paramspec=g_param_spec_enum(name, nick, desc,
+          GSTBT_TYPE_NOTE, GSTBT_NOTE_NONE, pspec_flags);
       break;
     case PT_SWITCH:
       //if(!(flags&GSTBT_PROPERTY_META_STATE)) {
@@ -618,36 +643,36 @@ GParamSpec *gstbml_register_param(GObjectClass *klass,gint prop_id, gint type, G
         if(min_val==-1) min_val=0;
         paramspec=g_param_spec_uint(name, nick, desc,
           min_val,max_val,def_val,
-          G_PARAM_READWRITE|GST_PARAM_CONTROLLABLE);
+          pspec_flags);
         */
         type=PT_ENUM;
         paramspec=g_param_spec_enum(name, nick, desc,
-          GSTBT_TYPE_TRIGGER_SWITCH, GSTGSTBT_TRIGGER_SWITCH_EMPTY,
-          G_PARAM_READWRITE|GST_PARAM_CONTROLLABLE);
+          GSTBT_TYPE_TRIGGER_SWITCH, GSTBT_TRIGGER_SWITCH_EMPTY,
+          pspec_flags);
       /*}
       else {
         // non-triggers use this
         //min_val=0;max_val=1;
         paramspec=g_param_spec_boolean(name, nick, desc,
           def_val,
-          G_PARAM_READWRITE|GST_PARAM_CONTROLLABLE);
+          pspec_flags);
       }*/
       break;
     case PT_BYTE:
-      // @todo gstreamer has no support for CHAR/UCHAR
+      // @todo: gstreamer has no support for CHAR/UCHAR
       paramspec=g_param_spec_uint(name, nick, desc,
         min_val,max_val,def_val,
-        G_PARAM_READWRITE|GST_PARAM_CONTROLLABLE);
+        pspec_flags);
       break;
     case PT_WORD:
       paramspec=g_param_spec_uint(name, nick, desc,
         min_val,max_val,def_val,
-        G_PARAM_READWRITE|GST_PARAM_CONTROLLABLE);
+        pspec_flags);
       break;
     case PT_ENUM:
       paramspec=g_param_spec_enum(name, nick, desc,
         enum_type, def_val,
-        G_PARAM_READWRITE|GST_PARAM_CONTROLLABLE);
+        pspec_flags);
       break;
     case PT_ATTR: // Attribute
       paramspec=g_param_spec_int(name, nick, desc,
@@ -659,6 +684,7 @@ GParamSpec *gstbml_register_param(GObjectClass *klass,gint prop_id, gint type, G
       break;
   }
   if(paramspec) {
+    // @todo: can we skip the gstbt_property_meta qdata business for some parameters
     g_param_spec_set_qdata(paramspec,gstbt_property_meta_quark,GINT_TO_POINTER(TRUE));
     g_param_spec_set_qdata(paramspec,gstbt_property_meta_quark_min_val, GINT_TO_POINTER(saved_min_val));
     g_param_spec_set_qdata(paramspec,gstbt_property_meta_quark_max_val, GINT_TO_POINTER(saved_max_val));
@@ -674,13 +700,19 @@ GParamSpec *gstbml_register_param(GObjectClass *klass,gint prop_id, gint type, G
 
 //-- common element functions
 
-void gstbml_set_param(gint type,gpointer addr,const GValue *value) {
+/**
+ * gstbml_set_param:
+ * @type: parameter type
+ * @addr: the address pointing to the value
+ * @value: the source
+ *
+ * Write a parameter according to @type to @addr from the @value.
+ */
+void gstbml_set_param(GstBMLParameterTypes type,gpointer addr,const GValue *value) {
   switch(type) {
-    case PT_NOTE: {
-      const gchar *str=g_value_get_string(value);
-      *(guint8 *)addr=(str && *str) ? gstbt_tone_conversion_note_string_2_number (str) : 0;
+    case PT_NOTE:
+      *(guint8 *)addr=(guint8)g_value_get_enum(value);
       break;
-    }
     case PT_SWITCH:
       *(guint8 *)addr=(guint8)g_value_get_boolean(value);
       //*(guint8 *)addr=(guint8)g_value_get_uint(value);
@@ -699,10 +731,18 @@ void gstbml_set_param(gint type,gpointer addr,const GValue *value) {
   }
 }
 
-void gstbml_get_param(gint type,gpointer addr,GValue *value) {
+/**
+ * gstbml_get_param:
+ * @type: parameter type
+ * @addr: the address pointing to the value
+ * @value: the target
+ *
+ * Read a parameter according to @type from @addr into the @value.
+ */
+void gstbml_get_param(GstBMLParameterTypes type,gpointer addr,GValue *value) {
   switch(type) {
     case PT_NOTE:
-      g_value_set_string(value, gstbt_tone_conversion_note_number_2_string ((guint)(*(guint8 *)addr)));
+      //g_value_set_string(value, gstbt_tone_conversion_note_number_2_string ((guint)(*(guint8 *)addr)));
       break;
     case PT_SWITCH:
       g_value_set_boolean(value,(guint)(*(guint8 *)addr));
@@ -722,8 +762,9 @@ void gstbml_get_param(gint type,gpointer addr,GValue *value) {
   }
 }
 
-/*
+/**
  * gstbml_calculate_buffer_frames:
+ * @bml: bml instance
  *
  * update the buffersize for calculation (in samples)
  * buffer_frames = samples_per_minute/ticks_per_minute
@@ -735,13 +776,19 @@ void gstbml_calculate_buffer_frames(GstBML *bml) {
   bml->ticktime=(GstClockTime)(0.5+((GST_SECOND*60.0)/ticks_per_minute));
 }
 
+/**
+ * gstbml_dispose:
+ * @bml: bml instance
+ *
+ * Dispose common buzzmachine data.
+ */
 void gstbml_dispose(GstBML *bml) {
   GList* node;
 
   if(bml->dispose_has_run) return;
   bml->dispose_has_run = TRUE;
 
-  GST_DEBUG("!!!! bml=%p",bml);
+  GST_DEBUG_OBJECT(bml->self,"!!!! bml=%p",bml);
 
   // unref list of voices
   if(bml->voices) {
@@ -756,3 +803,69 @@ void gstbml_dispose(GstBML *bml) {
   }
 }
 
+/**
+ * gstbml_fix_data:
+ * @elem: the element instance
+ * @buf: the data buffer
+ * @has_data: indication wheter the buffer has values != 0.0
+ *
+ * Fixes elements that output denormalized values (sets them to 0.0). Also
+ * adjust gap-flags. unfortunately this is quite expensive, although extra gap
+ * flags help.
+ */
+void gstbml_fix_data(GstElement *elem,GstBuffer *buf,gboolean has_data) {
+  BMLData *data=(BMLData *)GST_BUFFER_DATA(buf);
+  guint num_samples=GST_BUFFER_SIZE(buf)/sizeof(BMLData);
+  guint i;
+
+  if(has_data) {
+    has_data=FALSE;
+
+    // see also http://www.musicdsp.org/archive.php?classid=5#191
+    for(i=0;i<num_samples;i++) {
+      if(data[i]!=0.0) {
+        has_data=TRUE;
+        break;
+      }
+/* FIXME: we not configure the FPU to DAZ|FZ in libbuzztard-core
+ * this would not apply to other clients
+ */
+#if 0
+      /* isnormal checks for != zero */
+      if(G_LIKELY(isnormal(data[i]))) {
+        has_data=TRUE;
+        break;
+      }
+      else {
+        if(isnan(data[i])) {
+          GST_WARNING_OBJECT(elem,"data contains NaN");
+        }
+        else if(isinf(data[i])) {
+          GST_WARNING_OBJECT(elem,"data contains Inf");
+        }
+        else if(data[i]!=0.0) { //fpclassify(data[i])==FP_SUBNORMAL
+          GST_WARNING_OBJECT(bml_transform,"data contains Denormal");
+        }
+        data[i]=0.0;
+      }
+#endif
+    }
+#if 0
+    for(;i<num_samples;i++) {
+      if(G_UNLIKELY(!isnormal(data[i]))) {
+        data[i]=0.0;
+      }
+    }
+#endif
+  }
+  if(!has_data) {
+    GST_LOG_OBJECT(elem,"silent buffer");
+    GST_BUFFER_FLAG_SET(buf,GST_BUFFER_FLAG_GAP);
+  }
+  else {
+    // buzz generates relative loud output
+    gfloat fc=1.0/32768.0;
+    orc_scalarmultiply_f32_ns (data, data, fc, num_samples);
+    GST_BUFFER_FLAG_UNSET(buf,GST_BUFFER_FLAG_GAP);
+  }
+}
