@@ -219,7 +219,7 @@ static gboolean gst_bml_src_set_caps(GstBaseSrc *base, GstCaps *caps) {
     // @todo: irks, this resets all parameter to their default
     //bml(init(bml->bm,0,NULL));
   }
-  
+
   return ret;
 }
 
@@ -285,31 +285,19 @@ error:
   }
 }
 
-static void gst_bml_src_get_times(GstBaseSrc * base, GstBuffer * buffer, GstClockTime * start, GstClockTime * end) {
-  /* for live sources, sync on the timestamp of the buffer */
-  if (gst_base_src_is_live (base)) {
-    GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
+/*
+static gboolean gst_bml_src_start(GstBaseSrc * base) {
+  GstBMLSrc *bml_src=GST_BML_SRC(base);
 
-    if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
-      /* get duration to calculate end time */
-      GstClockTime duration = GST_BUFFER_DURATION (buffer);
-
-      if (GST_CLOCK_TIME_IS_VALID (duration)) {
-        *end = timestamp + duration;
-      }
-      *start = timestamp;
-    }
-  } else {
-    *start = -1;
-    *end = -1;
-  }
+  return TRUE;
 }
+*/
 
 static gboolean gst_bml_src_stop(GstBaseSrc * base) {
   GstBMLSrc *bml_src=GST_BML_SRC(base);
   GstBML *bml=GST_BML(bml_src);
   gpointer bm=bml->bm;
-  
+
   bml(stop(bm));
   return TRUE;
 }
@@ -319,26 +307,44 @@ static gboolean gst_bml_src_do_seek(GstBaseSrc * base, GstSegment * segment) {
   GstBML *bml=GST_BML(bml_src);
   GstClockTime time;
 
-  segment->time = segment->start;
   time = segment->last_stop;
+  bml->reverse = (segment->rate<0.0);
+  bml->running_time = time;
 
   /* now move to the time indicated */
   bml->n_samples = gst_util_uint64_scale_int(time, bml->samplerate, GST_SECOND);
-  bml->running_time = time;
 
-  g_assert (bml->running_time <= time);
-
-  if (GST_CLOCK_TIME_IS_VALID (segment->stop)) {
-    time = segment->stop;
-    bml->n_samples_stop = gst_util_uint64_scale_int(time, bml->samplerate,
-        GST_SECOND);
-    bml->check_seek_stop = TRUE;
-  } else {
-    bml->check_seek_stop = FALSE;
+  if (!bml->reverse) {
+    if (GST_CLOCK_TIME_IS_VALID (segment->start)) {
+      segment->time = segment->start;
+    }
+    if (GST_CLOCK_TIME_IS_VALID (segment->stop)) {
+      time = segment->stop;
+      bml->n_samples_stop = gst_util_uint64_scale_int(time, bml->samplerate,
+          GST_SECOND);
+      bml->check_eos = TRUE;
+    } else {
+      bml->check_eos = FALSE;
+    }
+  }
+  else {
+    if (GST_CLOCK_TIME_IS_VALID (segment->stop)) {
+      segment->time = segment->stop;
+    }
+    if (GST_CLOCK_TIME_IS_VALID (segment->start)) {
+      time = segment->start;
+      bml->n_samples_stop = gst_util_uint64_scale_int(time, bml->samplerate,
+          GST_SECOND);
+      bml->check_eos = TRUE;
+    } else {
+      bml->check_eos = FALSE;
+    }
   }
-  bml->seek_flags = segment->flags;
   bml->eos_reached = FALSE;
 
+  GST_DEBUG_OBJECT(bml_src,"seek from %"GST_TIME_FORMAT" to %"GST_TIME_FORMAT" cur %"GST_TIME_FORMAT" rate %lf",
+    GST_TIME_ARGS(segment->start),GST_TIME_ARGS(segment->stop),GST_TIME_ARGS(segment->last_stop),segment->rate);
+
   return TRUE;
 }
 
@@ -349,59 +355,83 @@ static gboolean gst_bml_src_is_seekable(GstBaseSrc * base) {
 
 static GstFlowReturn gst_bml_src_create_mono(GstBaseSrc *base, GstClockTime offset, guint length, GstBuffer ** buffer) {
   GstFlowReturn res;
+  GstPad *srcpad=GST_BASE_SRC_PAD(base);
   GstBMLSrc *bml_src=GST_BML_SRC(base);
   GstBMLSrcClass *klass=GST_BML_SRC_GET_CLASS(bml_src);
   GstBML *bml=GST_BML(bml_src);
-  GstBMLClass *bml_class=GST_BML_CLASS(klass);  
+  GstBMLClass *bml_class=GST_BML_CLASS(klass);
   GstBuffer *buf;
-  GstClockTime next_time;
+  GstClockTime next_running_time;
   gint64 n_samples;
   gdouble samples_done;
-  GstPad *srcpad=GST_BASE_SRC_PAD(base);
   BMLData *data,*seg_data;
   gpointer bm=bml->bm;
   guint todo,seg_size,samples_per_buffer;
   gboolean has_data;
+  gboolean partial_buffer=FALSE;
 
   if (G_UNLIKELY(bml->eos_reached)) {
-    GST_DEBUG("  EOS reached");
+    GST_DEBUG_OBJECT(bml_src,"EOS reached");
     return GST_FLOW_UNEXPECTED;
   }
 
   // the amount of samples to produce (handle rounding errors by collecting left over fractions)
-  samples_done = (gdouble)bml->running_time*(gdouble)bml->samplerate/(gdouble)GST_SECOND;
-  samples_per_buffer=(guint)(bml->samples_per_buffer+(samples_done-(gdouble)bml->n_samples));
-
-  /* check for eos */
-  if (bml->check_seek_stop &&
-    (bml->n_samples_stop > bml->n_samples) &&
-    (bml->n_samples_stop < bml->n_samples + samples_per_buffer)
-  ) {
-    /* calculate only partial buffer */
-    samples_per_buffer = bml->n_samples_stop - bml->n_samples;
-    n_samples = bml->n_samples_stop;
-    if (!(bml->seek_flags&GST_SEEK_FLAG_SEGMENT)) {
-      bml->eos_reached = TRUE;
-    }
+  samples_done=(gdouble)bml->running_time*(gdouble)bml->samplerate/(gdouble)GST_SECOND;
+  if (!bml->reverse) {
+    samples_per_buffer=(guint)(bml->samples_per_buffer+(samples_done-(gdouble)bml->n_samples));
   } else {
-    /* calculate full buffer */
-    n_samples = bml->n_samples + samples_per_buffer;
+    samples_per_buffer=(guint)(bml->samples_per_buffer+((gdouble)bml->n_samples-samples_done));
   }
 
-  next_time = gst_util_uint64_scale(n_samples,GST_SECOND,(guint64)bml->samplerate);
+  //GST_LOG_OBJECT(bml_src,"time=%"GST_TIME_FORMAT", samples_done=%lf, %"G_GINT64_FORMAT" spb=%u",
+  //  GST_TIME_ARGS(bml->running_time),samples_done,bml->n_samples,samples_per_buffer);
+
+  /* check for eos */
+  if (bml->check_eos) {
+    if (!bml->reverse) {
+      partial_buffer=((bml->n_samples_stop >= bml->n_samples) &&
+        (bml->n_samples_stop < bml->n_samples + samples_per_buffer));
+    } else {
+      partial_buffer=((bml->n_samples_stop < bml->n_samples) &&
+        (bml->n_samples_stop >= bml->n_samples - samples_per_buffer));
+    }
+  }
+
+  if (G_UNLIKELY(partial_buffer)) {
+    /* calculate only partial buffer */
+    samples_per_buffer = bml->n_samples_stop - bml->n_samples;
+    if(G_UNLIKELY(!samples_per_buffer)) {
+      GST_WARNING_OBJECT(bml_src,"0 samples left -> EOS reached");
+      bml->eos_reached = TRUE;
+      return GST_FLOW_UNEXPECTED;
+    }
+    n_samples = bml->n_samples_stop;
+    bml->eos_reached = TRUE;
+  } else {
+    /* calculate full buffer */
+    n_samples = bml->n_samples + (bml->reverse ? (-samples_per_buffer) : samples_per_buffer);
+  }
+  next_running_time = bml->running_time + (bml->reverse ? (-bml->ticktime) : bml->ticktime);
 
   /* allocate a new buffer suitable for this pad */
-  if ((res = gst_pad_alloc_buffer_and_set_caps (srcpad, bml->n_samples,
-      samples_per_buffer * sizeof(BMLData),
-      GST_PAD_CAPS (srcpad),
-      &buf)) != GST_FLOW_OK
-  ) {
+  res = gst_pad_alloc_buffer_and_set_caps (srcpad, bml->n_samples,
+    samples_per_buffer * sizeof(BMLData),
+    GST_PAD_CAPS (srcpad), &buf);
+  if (res != GST_FLOW_OK) {
     return res;
   }
 
-  GST_BUFFER_TIMESTAMP(buf)=bml->running_time;
-  GST_BUFFER_OFFSET_END(buf)=n_samples;
-  GST_BUFFER_DURATION(buf)=next_time - bml->running_time;
+  if (!bml->reverse) {
+    GST_BUFFER_TIMESTAMP(buf)=bml->running_time;
+    GST_BUFFER_DURATION(buf)=next_running_time-bml->running_time;
+    GST_BUFFER_OFFSET(buf)=bml->n_samples;
+    GST_BUFFER_OFFSET_END(buf)=n_samples;
+  } else {
+    GST_BUFFER_TIMESTAMP(buf)=next_running_time;
+    GST_BUFFER_DURATION(buf)=bml->running_time-next_running_time;
+    GST_BUFFER_OFFSET(buf)=n_samples;
+    GST_BUFFER_OFFSET_END(buf)=bml->n_samples;
+  }
 
   /* @todo: split up processing of the buffer into chunks so that params can
    * be updated when required (e.g. for the subticks-feature).
@@ -410,11 +440,10 @@ static GstFlowReturn gst_bml_src_create_mono(GstBaseSrc *base, GstClockTime offs
   bml(gstbml_sync_values(bml,bml_class));
   bml(tick(bm));
 
-  bml->running_time += bml->ticktime;
-  //bml->running_time = next_time;
+  bml->running_time = next_running_time;
   bml->n_samples = n_samples;
 
-  GST_DEBUG("  calling work(%d)",samples_per_buffer);
+  GST_DEBUG_OBJECT(bml_src,"  calling work(%d)",samples_per_buffer);
   data = (BMLData *)GST_BUFFER_DATA(buf);
   // some buzzmachines expect a cleared buffer
   //for(i=0;i<samples_per_buffer;i++) data[i]=0.0f;
@@ -425,23 +454,13 @@ static GstFlowReturn gst_bml_src_create_mono(GstBaseSrc *base, GstClockTime offs
   has_data=FALSE;
   while (todo) {
     // 256 is MachineInterface.h::MAX_BUFFER_LENGTH
-    seg_size = (todo>256) ? 256 : todo; 
+    seg_size = (todo>256) ? 256 : todo;
     /* mode does not really matter for generators */
     has_data |= bml(work(bm,seg_data,(int)seg_size,2/*WM_WRITE*/));
     seg_data = &seg_data[seg_size];
     todo -= seg_size;
   }
-  if(!has_data) {
-    GST_INFO_OBJECT(bml_src,"silent buffer");
-    GST_BUFFER_FLAG_SET(buf,GST_BUFFER_FLAG_GAP);
-  }
-  else {
-    // buzz generates relative loud output
-    //guint i;
-    //for(i=0;i<samples_per_buffer;i++) data[i]/=32768.0f;
-    gfloat fc=1.0/32768.0;
-    oil_scalarmultiply_f32_ns (data, data, &fc, samples_per_buffer);
-  }
+  gstbml_fix_data((GstElement*)bml_src,buf,has_data);
 
   // return results
   *buffer = buf;
@@ -454,9 +473,9 @@ static GstFlowReturn gst_bml_src_create_stereo(GstBaseSrc *base, GstClockTime of
   GstBMLSrc *bml_src=GST_BML_SRC(base);
   GstBMLSrcClass *klass=GST_BML_SRC_GET_CLASS(bml_src);
   GstBML *bml=GST_BML(bml_src);
-  GstBMLClass *bml_class=GST_BML_CLASS(klass);  
+  GstBMLClass *bml_class=GST_BML_CLASS(klass);
   GstBuffer *buf;
-  GstClockTime next_time;
+  GstClockTime next_running_time;
   gint64 n_samples;
   gdouble samples_done;
   GstPad *srcpad=GST_BASE_SRC_PAD(base);
@@ -464,44 +483,70 @@ static GstFlowReturn gst_bml_src_create_stereo(GstBaseSrc *base, GstClockTime of
   gpointer bm=bml->bm;
   guint todo,seg_size,samples_per_buffer;
   gboolean has_data;
+  gboolean partial_buffer=FALSE;
 
   if (G_UNLIKELY(bml->eos_reached)) {
-    GST_DEBUG("  EOS reached");
+    GST_WARNING_OBJECT(bml_src,"EOS reached");
     return GST_FLOW_UNEXPECTED;
   }
 
   // the amount of samples to produce (handle rounding errors by collecting left over fractions)
-  samples_done = (gdouble)bml->running_time*(gdouble)bml->samplerate/(gdouble)GST_SECOND;
-  samples_per_buffer=(guint)(bml->samples_per_buffer+(samples_done-(gdouble)bml->n_samples));
+  samples_done=(gdouble)bml->running_time*(gdouble)bml->samplerate/(gdouble)GST_SECOND;
+  if (!bml->reverse) {
+    samples_per_buffer=(guint)(bml->samples_per_buffer+(samples_done-(gdouble)bml->n_samples));
+  } else {
+    samples_per_buffer=(guint)(bml->samples_per_buffer+((gdouble)bml->n_samples-samples_done));
+  }
+
+  //GST_LOG_OBJECT(bml_src,"time=%"GST_TIME_FORMAT", samples_done=%lf, %"G_GINT64_FORMAT" spb=%u",
+  //  GST_TIME_ARGS(bml->running_time),samples_done,bml->n_samples,samples_per_buffer);
 
   /* check for eos */
-  if (bml->check_seek_stop &&
-    (bml->n_samples_stop > bml->n_samples) &&
-    (bml->n_samples_stop < bml->n_samples + samples_per_buffer)
-  ) {
+  if (bml->check_eos) {
+    if (!bml->reverse) {
+      partial_buffer=((bml->n_samples_stop >= bml->n_samples) &&
+        (bml->n_samples_stop < bml->n_samples + samples_per_buffer));
+    } else {
+      partial_buffer=((bml->n_samples_stop < bml->n_samples) &&
+        (bml->n_samples_stop >= bml->n_samples - samples_per_buffer));
+    }
+  }
+
+  if (G_UNLIKELY(partial_buffer)) {
     /* calculate only partial buffer */
     samples_per_buffer = bml->n_samples_stop - bml->n_samples;
+    if(G_UNLIKELY(!samples_per_buffer)) {
+      GST_WARNING_OBJECT(bml_src,"0 samples left -> EOS reached");
+      bml->eos_reached = TRUE;
+      return GST_FLOW_UNEXPECTED;
+    }
     n_samples = bml->n_samples_stop;
     bml->eos_reached = TRUE;
   } else {
     /* calculate full buffer */
-    n_samples = bml->n_samples + samples_per_buffer;
+    n_samples = bml->n_samples + (bml->reverse ? (-samples_per_buffer) : samples_per_buffer);
   }
-
-  next_time = gst_util_uint64_scale(n_samples,GST_SECOND,(guint64)bml->samplerate);
+  next_running_time = bml->running_time + (bml->reverse ? (-bml->ticktime) : bml->ticktime);
 
   /* allocate a new buffer suitable for this pad */
-  if ((res = gst_pad_alloc_buffer_and_set_caps (srcpad, bml->n_samples,
+  res = gst_pad_alloc_buffer_and_set_caps (srcpad, bml->n_samples,
       samples_per_buffer * 2 * sizeof(BMLData),
-      GST_PAD_CAPS (srcpad),
-      &buf)) != GST_FLOW_OK
-  ) {
+      GST_PAD_CAPS (srcpad), &buf);
+  if (res != GST_FLOW_OK) {
     return res;
   }
 
-  GST_BUFFER_TIMESTAMP(buf)=bml->running_time;
-  GST_BUFFER_OFFSET_END(buf)=n_samples;
-  GST_BUFFER_DURATION(buf)=next_time - bml->running_time;
+  if (!bml->reverse) {
+    GST_BUFFER_TIMESTAMP(buf)=bml->running_time;
+    GST_BUFFER_DURATION(buf)=next_running_time-bml->running_time;
+    GST_BUFFER_OFFSET(buf)=bml->n_samples;
+    GST_BUFFER_OFFSET_END(buf)=n_samples;
+  } else {
+    GST_BUFFER_TIMESTAMP(buf)=next_running_time;
+    GST_BUFFER_DURATION(buf)=bml->running_time-next_running_time;
+    GST_BUFFER_OFFSET(buf)=n_samples;
+    GST_BUFFER_OFFSET_END(buf)=bml->n_samples;
+  }
 
   /* @todo: split up processing of the buffer into chunks so that params can
    * be updated when required (e.g. for the subticks-feature).
@@ -510,11 +555,10 @@ static GstFlowReturn gst_bml_src_create_stereo(GstBaseSrc *base, GstClockTime of
   bml(gstbml_sync_values(bml,bml_class));
   bml(tick(bm));
 
-  bml->running_time += bml->ticktime;
-  //bml->running_time = next_time;
+  bml->running_time = next_running_time;
   bml->n_samples = n_samples;
 
-  GST_DEBUG("  calling work_m2s(%d)",samples_per_buffer);
+  GST_DEBUG_OBJECT(bml_src,"  calling work_m2s(%d)",samples_per_buffer);
   data=(BMLData *)GST_BUFFER_DATA(buf);
   // some buzzmachines expect a cleared buffer
   //for(i=0;i<samples_per_buffer*2;i++) data[i]=0.0;
@@ -525,23 +569,13 @@ static GstFlowReturn gst_bml_src_create_stereo(GstBaseSrc *base, GstClockTime of
   has_data=FALSE;
   while (todo) {
     // 256 is MachineInterface.h::MAX_BUFFER_LENGTH
-    seg_size = (todo>256) ? 256 : todo; 
+    seg_size = (todo>256) ? 256 : todo;
     /* mode does not really matter for generators, NULL for input should be fine */
     has_data |= bml(work_m2s(bm,NULL,seg_data,(int)seg_size,2/*WM_WRITE*/));
     seg_data = &seg_data[seg_size*2];
     todo -= seg_size;
   }
-  if(!has_data) {
-    GST_INFO_OBJECT(bml_src,"silent buffer");
-    GST_BUFFER_FLAG_SET(buf,GST_BUFFER_FLAG_GAP);
-  }
-  else {
-    // buzz generates relative loud output
-    //guint i;
-    //for(i=0;i<samples_per_buffer*2;i++) data[i]/=32768.0;
-    gfloat fc=1.0/32768.0;
-    oil_scalarmultiply_f32_ns (data, data, &fc, samples_per_buffer*2);
-  }
+  gstbml_fix_data((GstElement*)bml_src,buf,has_data);
 
   // return results
   *buffer = buf;
@@ -573,9 +607,7 @@ static void gst_bml_src_dispose(GObject *object) {
 
   gstbml_dispose(bml);
 
-  if(G_OBJECT_CLASS(parent_class)->dispose) {
-    (G_OBJECT_CLASS(parent_class)->dispose)(object);
-  }
+  G_OBJECT_CLASS(parent_class)->dispose(object);
 }
 
 static void gst_bml_src_finalize(GObject *object) {
@@ -584,9 +616,7 @@ static void gst_bml_src_finalize(GObject *object) {
 
   bml(gstbml_finalize(bml));
 
-  if(G_OBJECT_CLASS(parent_class)->finalize) {
-    (G_OBJECT_CLASS(parent_class)->finalize)(object);
-  }
+  G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
 static void gst_bml_src_base_finalize(GstBMLSrcClass *klass) {
@@ -612,7 +642,7 @@ static void gst_bml_src_init(GstBMLSrc *bml_src) {
 
   //bml(gstbml_pads(GST_ELEMENT(bml_src),bml,gst_bml_src_link));
   srcpad=GST_BASE_SRC_PAD(bml_src);
-  gst_pad_set_fixatecaps_function (srcpad, gst_bml_src_src_fixate);
+  gst_pad_set_fixatecaps_function(srcpad, gst_bml_src_src_fixate);
 
   GST_DEBUG("  done");
 }
@@ -634,7 +664,7 @@ static void gst_bml_src_class_init(GstBMLSrcClass * klass) {
   gstbasesrc_class->is_seekable = GST_DEBUG_FUNCPTR(gst_bml_src_is_seekable);
   gstbasesrc_class->do_seek     = GST_DEBUG_FUNCPTR(gst_bml_src_do_seek);
   gstbasesrc_class->query       = GST_DEBUG_FUNCPTR(gst_bml_src_query);
-  gstbasesrc_class->get_times   = GST_DEBUG_FUNCPTR(gst_bml_src_get_times);
+  //gstbasesrc_class->start       = GST_DEBUG_FUNCPTR(gst_bml_src_start);
   gstbasesrc_class->stop        = GST_DEBUG_FUNCPTR(gst_bml_src_stop);
   if(bml_class->output_channels==1) {
     gstbasesrc_class->create    = GST_DEBUG_FUNCPTR(gst_bml_src_create_mono);
@@ -652,21 +682,27 @@ static void gst_bml_src_base_init(GstBMLSrcClass *klass) {
   GstElementClass *element_class=GST_ELEMENT_CLASS(klass);
   //GstPadTemplate *templ;
   gpointer bmh;
+  static GstPadTemplate *mono_src_pad_template=NULL;
+  static GstPadTemplate *stereo_src_pad_template=NULL;
 
   GST_INFO("initializing base");
 
   bmh=bml(gstbml_class_base_init(bml_class,G_TYPE_FROM_CLASS(klass),1,0));
 
   if(bml_class->output_channels==1) {
-    gst_element_class_add_pad_template(element_class, gst_static_pad_template_get (&bml_pad_caps_mono_src_template));
+    if(G_UNLIKELY(!mono_src_pad_template))
+      mono_src_pad_template=gst_static_pad_template_get(&bml_pad_caps_mono_src_template);
+    gst_element_class_add_pad_template(element_class,mono_src_pad_template);
     GST_INFO("  added mono src pad template");
   }
   else {
-    gst_element_class_add_pad_template(element_class, gst_static_pad_template_get (&bml_pad_caps_stereo_src_template));
+    if(G_UNLIKELY(!stereo_src_pad_template))
+      stereo_src_pad_template=gst_static_pad_template_get(&bml_pad_caps_stereo_src_template);
+    gst_element_class_add_pad_template(element_class,stereo_src_pad_template);
     GST_INFO("  added stereo src pad template");
   }
 
-  bml(gstbml_class_set_details(element_class,bmh,"Source/Audio/BML"));
+  bml(gstbml_class_set_details(element_class,bml_class,bmh,"Source/Audio/BML"));
 }
 
 
@@ -702,11 +738,13 @@ GType bml(src_get_type(const char *element_type_name, gboolean is_polyphonic, gb
     NULL,               /* interface_finalize */
     NULL                /* interface_data */
   };
+#if !GST_CHECK_VERSION(0,10,31)
   const GInterfaceInfo help_interface_info = {
     NULL,               /* interface_init */
     NULL,               /* interface_finalize */
     NULL                /* interface_data */
   };
+#endif
   const GInterfaceInfo preset_interface_info = {
     (GInterfaceInitFunc) gst_bml_preset_interface_init,        /* interface_init */
     NULL,               /* interface_finalize */
@@ -727,10 +765,12 @@ GType bml(src_get_type(const char *element_type_name, gboolean is_polyphonic, gb
     g_type_add_interface_static(element_type, GST_TYPE_CHILD_PROXY, &child_proxy_interface_info);
     g_type_add_interface_static(element_type, GSTBT_TYPE_CHILD_BIN, &child_bin_interface_info);
   }
+#if !GST_CHECK_VERSION(0,10,31)
   // check if this plugin has user docs
   if(has_help) {
     g_type_add_interface_static(element_type, GSTBT_TYPE_HELP, &help_interface_info);
   }
+#endif
   // add presets iface
   g_type_add_interface_static(element_type, GST_TYPE_PRESET, &preset_interface_info);
 
