@@ -232,7 +232,7 @@ static gboolean gst_bml_transform_set_caps(GstBaseTransform * base, GstCaps * in
 
   GST_DEBUG("set_caps: in %"GST_PTR_FORMAT"  out %"GST_PTR_FORMAT, incaps, outcaps);
   structure=gst_caps_get_structure(incaps,0);
-  
+
   if((ret = gst_structure_get_int(structure, "rate", &bml->samplerate)) && (samplerate!=bml->samplerate)) {
     bml(set_master_info(bml->beats_per_minute,bml->ticks_per_beat,bml->samplerate));
     // @todo: irks, this resets all parameter to their default
@@ -274,33 +274,22 @@ static GstFlowReturn gst_bml_transform_transform_ip_mono(GstBaseTransform *base,
   }
   else {
     // buzz generates loud output
-    //for(i=0;i<samples_per_buffer;i++) data[i]*=32768.0f;
     gfloat fc=32768.0;
-    oil_scalarmultiply_f32_ns (data, data, &fc, samples_per_buffer);
+    orc_scalarmultiply_f32_ns (data, data, fc, samples_per_buffer);
   }
 
-  GST_DEBUG("  calling work(%d,%d)",samples_per_buffer,mode);
+  GST_DEBUG_OBJECT(bml_transform,"  calling work(%d,%d)",samples_per_buffer,mode);
   todo = samples_per_buffer;
   seg_data = data;
   has_data=FALSE;
   while (todo) {
     // 256 is MachineInterface.h::MAX_BUFFER_LENGTH
-    seg_size = (todo>256) ? 256 : todo; 
+    seg_size = (todo>256) ? 256 : todo;
     has_data |= bml(work(bm,seg_data,(int)seg_size,mode));
     seg_data = &seg_data[seg_size];
     todo -= seg_size;
   }
-  if(!has_data) {
-    GST_INFO_OBJECT(bml_transform,"silent buffer");
-    GST_BUFFER_FLAG_SET(outbuf,GST_BUFFER_FLAG_GAP);
-  }
-  else {
-    GST_BUFFER_FLAG_UNSET(outbuf,GST_BUFFER_FLAG_GAP);
-    // buzz generates relative loud output
-    //for(i=0;i<samples_per_buffer;i++) data[i]/=32768.0f;
-    gfloat fc=1.0/32768.0;
-    oil_scalarmultiply_f32_ns (data, data, &fc, samples_per_buffer);
-  }
+  gstbml_fix_data((GstElement*)bml_transform,outbuf,has_data);
 
   return(GST_FLOW_OK);
 }
@@ -336,12 +325,11 @@ static GstFlowReturn gst_bml_transform_transform_ip_stereo(GstBaseTransform *bas
     mode=2; /* WM_WRITE */
   }
   else {
-    //for(i=0;i<samples_per_buffer*2;i++) data[i]*=32768.0;
     gfloat fc=32768.0;
-    oil_scalarmultiply_f32_ns (data, data, &fc, samples_per_buffer*2);
+    orc_scalarmultiply_f32_ns (data, data, fc, samples_per_buffer*2);
   }
 
-  GST_DEBUG("  calling work_m2s(%d,%d)",samples_per_buffer,mode);
+  GST_DEBUG_OBJECT(bml_transform,"  calling work_m2s(%d,%d)",samples_per_buffer,mode);
   todo = samples_per_buffer;
   seg_data = data;
   has_data=FALSE;
@@ -353,16 +341,7 @@ static GstFlowReturn gst_bml_transform_transform_ip_stereo(GstBaseTransform *bas
     seg_data = &seg_data[seg_size*2];
     todo -= seg_size;
   }
-  if(!has_data) {
-    GST_INFO_OBJECT(bml_transform,"silent buffer");
-    GST_BUFFER_FLAG_SET(outbuf,GST_BUFFER_FLAG_GAP);
-  }
-  else {
-    GST_BUFFER_FLAG_UNSET(outbuf,GST_BUFFER_FLAG_GAP);
-    //for(i=0;i<samples_per_buffer*2;i++) data[i]/=32768.0;
-    gfloat fc=1.0/32768.0;
-    oil_scalarmultiply_f32_ns (data, data, &fc, samples_per_buffer*2);
-  }
+  gstbml_fix_data((GstElement*)bml_transform,outbuf,has_data);
 
   return(GST_FLOW_OK);
 }
@@ -382,12 +361,12 @@ static GstFlowReturn gst_bml_transform_transform_mono_to_stereo(GstBaseTransform
   datai=(BMLData *)GST_BUFFER_DATA (inbuf);
   datao=(BMLData *)GST_BUFFER_DATA (outbuf);
   samples_per_buffer=GST_BUFFER_SIZE(inbuf)/sizeof(BMLData);
-  
+
   // some buzzmachines expect a cleared buffer
   //for(i=0;i<samples_per_buffer*2;i++) datao[i]=0.0f;
   memset(datao,0,samples_per_buffer*2*sizeof(BMLData));
-  
-  GST_DEBUG("input : %p,%d  output: %p,%d",
+
+  GST_DEBUG_OBJECT(bml_transform,"input : %p,%d  output: %p,%d",
     datai,GST_BUFFER_SIZE(inbuf),
     datao,GST_BUFFER_SIZE(outbuf));
 
@@ -402,7 +381,7 @@ static GstFlowReturn gst_bml_transform_transform_mono_to_stereo(GstBaseTransform
   if (gst_base_transform_is_passthrough (base)) {
     // we would actually need to convert mono to stereo here
     // but this is not even called
-    GST_WARNING("m2s in passthrough mode");
+    GST_WARNING_OBJECT(bml_transform,"m2s in passthrough mode");
     //return GST_FLOW_OK;
   }
   /**/
@@ -412,34 +391,24 @@ static GstFlowReturn gst_bml_transform_transform_mono_to_stereo(GstBaseTransform
     mode=2; /* WM_WRITE */
   }
   else {
-    //for(i=0;i<samples_per_buffer;i++) datai[i]*=32768.0;
     gfloat fc=32768.0;
-    oil_scalarmultiply_f32_ns (datai, datai, &fc, samples_per_buffer);
+    orc_scalarmultiply_f32_ns (datai, datai, fc, samples_per_buffer);
   }
 
-  GST_DEBUG("  calling work_m2s(%d,%d)",samples_per_buffer,mode);
+  GST_DEBUG_OBJECT(bml_transform,"  calling work_m2s(%d,%d)",samples_per_buffer,mode);
   todo = samples_per_buffer;
   seg_datai = datai;
   seg_datao = datao;
   has_data = FALSE;
   while (todo) {
     // 256 is MachineInterface.h::MAX_BUFFER_LENGTH
-    seg_size = (todo>256) ? 256 : todo; 
+    seg_size = (todo>256) ? 256 : todo;
     has_data |= bml(work_m2s(bm,seg_datai,seg_datao,(int)seg_size,mode));
     seg_datai = &seg_datai[seg_size];
     seg_datao = &seg_datao[seg_size*2];
     todo -= seg_size;
   }
-  if(!has_data) {
-    GST_INFO_OBJECT(bml_transform,"silent buffer");
-    GST_BUFFER_FLAG_SET(outbuf,GST_BUFFER_FLAG_GAP);
-  }
-  else {
-    GST_BUFFER_FLAG_UNSET(outbuf,GST_BUFFER_FLAG_GAP);
-    //for(i=0;i<samples_per_buffer;i++) datao[i]/=32768.0;
-    gfloat fc=1.0/32768.0;
-    oil_scalarmultiply_f32_ns (datao, datao, &fc, samples_per_buffer*2);
-  }
+  gstbml_fix_data((GstElement*)bml_transform,outbuf,has_data);
 
   return(GST_FLOW_OK);
 }
@@ -475,12 +444,12 @@ static GstCaps *gst_bml_transform_transform_caps(GstBaseTransform * base, GstPad
   /* if we should produce this output, what can we accept */
   if (direction == GST_PAD_SRC) {
     GST_INFO_OBJECT (base, "allow %d input channel", bml_class->input_channels);
-    gst_structure_set (structure, 
+    gst_structure_set (structure,
       "channels", G_TYPE_INT, bml_class->input_channels,
       NULL);
   } else {
     GST_INFO_OBJECT (base, "allow %d output channels", bml_class->output_channels);
-    gst_structure_set (structure, 
+    gst_structure_set (structure,
       "channels", G_TYPE_INT, bml_class->output_channels,
       NULL);
   }
@@ -531,7 +500,7 @@ static gboolean gst_bml_transform_start(GstBaseTransform * base) {
         GST_WARNING("unhandled type : %d",type);
     }
   }
-  
+
   return TRUE;
 }
 #endif
@@ -540,7 +509,7 @@ static gboolean gst_bml_transform_stop(GstBaseTransform * base) {
   GstBMLTransform *bml_transform=GST_BML_TRANSFORM(base);
   GstBML *bml=GST_BML(bml_transform);
   gpointer bm=bml->bm;
-  
+
   bml(stop(bm));
   return TRUE;
 }
@@ -569,9 +538,7 @@ static void gst_bml_transform_dispose(GObject *object) {
 
   gstbml_dispose(bml);
 
-  if(G_OBJECT_CLASS(parent_class)->dispose) {
-    (G_OBJECT_CLASS(parent_class)->dispose)(object);
-  }
+  G_OBJECT_CLASS(parent_class)->dispose(object);
 }
 
 static void gst_bml_transform_finalize(GObject *object) {
@@ -580,9 +547,7 @@ static void gst_bml_transform_finalize(GObject *object) {
 
   bml(gstbml_finalize(bml));
 
-  if(G_OBJECT_CLASS(parent_class)->finalize) {
-    (G_OBJECT_CLASS(parent_class)->finalize)(object);
-  }
+  G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
 static void gst_bml_transform_base_finalize(GstBMLTransformClass *klass) {
@@ -665,29 +630,41 @@ static void gst_bml_transform_base_init(GstBMLTransformClass *klass) {
   GstElementClass *element_class=GST_ELEMENT_CLASS(klass);
   //GstPadTemplate *templ;
   gpointer bmh;
+  static GstPadTemplate *mono_src_pad_template=NULL;
+  static GstPadTemplate *stereo_src_pad_template=NULL;
+  static GstPadTemplate *mono_sink_pad_template=NULL;
+  static GstPadTemplate *stereo_sink_pad_template=NULL;
 
   GST_INFO("initializing base");
 
   bmh=bml(gstbml_class_base_init(bml_class,G_TYPE_FROM_CLASS(klass),1,1));
 
   if(bml_class->output_channels==1) {
-    gst_element_class_add_pad_template(element_class,gst_static_pad_template_get(&bml_pad_caps_mono_src_template));
+    if(G_UNLIKELY(!mono_src_pad_template))
+      mono_src_pad_template=gst_static_pad_template_get(&bml_pad_caps_mono_src_template);
+    gst_element_class_add_pad_template(element_class,mono_src_pad_template);
     GST_INFO("  added mono src pad template");
   }
   else {
-    gst_element_class_add_pad_template(element_class,gst_static_pad_template_get(&bml_pad_caps_stereo_src_template));
+    if(G_UNLIKELY(!stereo_src_pad_template))
+      stereo_src_pad_template=gst_static_pad_template_get(&bml_pad_caps_stereo_src_template);
+    gst_element_class_add_pad_template(element_class,stereo_src_pad_template);
     GST_INFO("  added stereo src pad template");
   }
   if(bml_class->input_channels==1) {
-    gst_element_class_add_pad_template(element_class,gst_static_pad_template_get(&bml_pad_caps_mono_sink_template));
+    if(G_UNLIKELY(!mono_sink_pad_template))
+      mono_sink_pad_template=gst_static_pad_template_get(&bml_pad_caps_mono_sink_template);
+    gst_element_class_add_pad_template(element_class,mono_sink_pad_template);
     GST_INFO("  added mono sink pad template");
   }
   else {
-    gst_element_class_add_pad_template(element_class,gst_static_pad_template_get(&bml_pad_caps_stereo_sink_template));
+    if(G_UNLIKELY(!stereo_sink_pad_template))
+      stereo_sink_pad_template=gst_static_pad_template_get(&bml_pad_caps_stereo_sink_template);
+    gst_element_class_add_pad_template(element_class,stereo_sink_pad_template);
     GST_INFO("  added stereo sink pad template");
   }
 
-  bml(gstbml_class_set_details(element_class,bmh,"Filter/Effect/Audio/BML"));
+  bml(gstbml_class_set_details(element_class,bml_class,bmh,"Filter/Effect/Audio/BML"));
 }
 
 
@@ -723,11 +700,13 @@ GType bml(transform_get_type(const char *element_type_name, gboolean is_polyphon
     NULL,               /* interface_finalize */
     NULL                /* interface_data */
   };
+#if !GST_CHECK_VERSION(0,10,31)
   const GInterfaceInfo help_interface_info = {
     NULL,               /* interface_init */
     NULL,               /* interface_finalize */
     NULL                /* interface_data */
   };
+#endif
   const GInterfaceInfo preset_interface_info = {
     (GInterfaceInitFunc) gst_bml_preset_interface_init,        /* interface_init */
     NULL,               /* interface_finalize */
@@ -748,10 +727,12 @@ GType bml(transform_get_type(const char *element_type_name, gboolean is_polyphon
     g_type_add_interface_static(element_type, GST_TYPE_CHILD_PROXY, &child_proxy_interface_info);
     g_type_add_interface_static(element_type, GSTBT_TYPE_CHILD_BIN, &child_bin_interface_info);
   }
+#if !GST_CHECK_VERSION(0,10,31)
   // check if this plugin has user docs
   if(has_help) {
     g_type_add_interface_static(element_type, GSTBT_TYPE_HELP, &help_interface_info);
   }
+#endif
   // add presets iface
   g_type_add_interface_static(element_type, GST_TYPE_PRESET, &preset_interface_info);
 
