@@ -158,7 +158,7 @@ gboolean bml(gstbml_register_element(GstPlugin *plugin, GstStructure *bml_meta))
     voice_type = bml(v_get_type(voice_type_name));
     GST_INFO("  voice type \"%s\" is 0x%lu", voice_type_name,(gulong)voice_type);
   }
-    
+
   // create the element type
   switch(type) {
     case MT_MASTER: // (Sink)
@@ -201,9 +201,9 @@ gpointer bml(gstbml_class_base_init(GstBMLClass *klass, GType type, gint numsrcp
   const gchar *dll_name;
 
   GST_INFO("initializing base: type=0x%lu",(gulong)type);
-  
+
   dll_name=(gchar*)gst_structure_get_string(bml_meta,"plugin-filename");
-  
+
   klass->dll_name=g_filename_from_utf8(dll_name,-1,NULL,NULL,NULL);
   klass->help_uri=(gchar*)gst_structure_get_string(bml_meta,"help-filename");
   klass->preset_path=(gchar*)gst_structure_get_string(bml_meta,"preset-filename");
@@ -211,9 +211,9 @@ gpointer bml(gstbml_class_base_init(GstBMLClass *klass, GType type, gint numsrcp
 
   bmh=bml(open(klass->dll_name));
   g_assert(bmh);
-    
+
   GST_INFO("  bmh=0x%p",bmh);
-  
+
   /* we now need to ensure that gst_bmlv_class_init() get bmh */
   if(voice_type_name) {
     GST_INFO("prepare voice-type %s",voice_type_name);
@@ -223,7 +223,7 @@ gpointer bml(gstbml_class_base_init(GstBMLClass *klass, GType type, gint numsrcp
     //g_hash_table_insert(bml_descriptors_by_voice_type,GINT_TO_POINTER(voice_type),(gpointer)bmh);
     g_type_class_ref(voice_type);
   }
-  
+
   GST_INFO("initializing base: bmh=0x%p, dll_name=%s, voice_type=0x%lu",bmh,((klass->dll_name)?klass->dll_name:"?"),(gulong)voice_type);
 
   klass->bmh=bmh;
@@ -247,7 +247,11 @@ gpointer bml(gstbml_class_base_init(GstBMLClass *klass, GType type, gint numsrcp
   return(bmh);
 }
 
+/* it see that this is never called :/
+ * due to that we 'leak' one instance of every buzzmachine we loaded
+ */
 void bml(gstbml_base_finalize(GstBMLClass *klass)) {
+  GST_INFO("!!!! klass=%p",klass);
   bml(close(klass->bmh));
   g_free(klass->dll_name);
 }
@@ -257,7 +261,7 @@ void bml(gstbml_base_finalize(GstBMLClass *klass)) {
  *
  * Get metadata from buzz-machine and set as element details
  */
-void bml(gstbml_class_set_details(GstElementClass *klass, gpointer bmh, const gchar *category)) {
+void bml(gstbml_class_set_details(GstElementClass *klass, GstBMLClass *bml_class, gpointer bmh, const gchar *category)) {
   GstElementDetails details;
   gchar *str;
   GType type=G_TYPE_FROM_CLASS(klass);
@@ -266,7 +270,7 @@ void bml(gstbml_class_set_details(GstElementClass *klass, gpointer bmh, const gc
   const gchar *extra_categories=gst_structure_get_string(bml_meta,"categories");
 
   /* construct the element details struct */
-  // @todo: do we want different charsets for BML_WRAPPED/BML_NATIVE? 
+  // @todo: do we want different charsets for BML_WRAPPED/BML_NATIVE?
   bml(get_machine_info(bmh,BM_PROP_SHORT_NAME,(void *)&str));
   details.longname=g_convert(str,-1,"UTF-8","WINDOWS-1252",NULL,NULL,NULL);
   bml(get_machine_info(bmh,BM_PROP_NAME,(void *)&str));
@@ -285,6 +289,11 @@ void bml(gstbml_class_set_details(GstElementClass *klass, gpointer bmh, const gc
   g_free(details.description);
   g_free(details.author);
   g_free(details.klass);
+#if GST_CHECK_VERSION(0,10,31)
+  if(bml_class->help_uri) {
+    gst_element_class_set_documentation_uri (klass, bml_class->help_uri);
+  }
+#endif
   GST_DEBUG("  element_class details have been set");
 }
 
@@ -323,7 +332,7 @@ static GType gst_bml_register_global_enum_type(GObjectClass *klass, gpointer bmh
       //  count=50;
       //}
       // DEBUG
-      
+
       // some plugins just have text for only val=min/max
       // don't make an enum for those
       //if(total-tcount<=2) {
@@ -350,7 +359,7 @@ static GType gst_bml_register_global_enum_type(GObjectClass *klass, gpointer bmh
         enums[k].value=0;
         enums[k].value_name=NULL;
         enums[k].value_nick=NULL;
-  
+
         enum_type = g_enum_register_static (type_name, enums);
         GST_INFO("register enum '%s' with %d values",type_name,vcount);
       }
@@ -372,7 +381,7 @@ GType bml(gstbml_register_track_enum_type(GObjectClass *klass, gpointer bmh, gin
 
   desc=bml(describe_track_value(bmh, i, min_val));
   GST_INFO("check enum, description= '%s', (entries=(%d-%d)=%d), no_val=%d",desc,max_val,min_val,((max_val+1)-min_val),no_val);
-  
+
   //if(desc && g_ascii_isalpha(desc[0])) {
     gchar *type_name;
     const gchar *class_type_name;
@@ -427,7 +436,7 @@ GType bml(gstbml_register_track_enum_type(GObjectClass *klass, gpointer bmh, gin
         enums[k].value=0;
         enums[k].value_name=NULL;
         enums[k].value_nick=NULL;
-  
+
         enum_type = g_enum_register_static (type_name, enums);
         GST_INFO("register enum '%s' with %d values",type_name,vcount);
       }
@@ -467,15 +476,17 @@ void bml(gstbml_class_prepare_properties(GObjectClass *klass, GstBMLClass *bml_c
     "host-callbacks property",
     "Buzz host callback structure",
     G_PARAM_WRITABLE));
-    
+
   if(bml(gstbml_is_polyphonic(bmh))) {
     g_object_class_override_property(klass, prop_id, "children");
     prop_id++;
   }
+#if !GST_CHECK_VERSION(0,10,31)
   if(bml_class->help_uri) {
     g_object_class_override_property(klass, prop_id, "documentation-uri");
     prop_id++;
   }
+#endif
 
   // register attributes as gobject properties
   if(bml(get_machine_info(bmh,BM_PROP_NUM_ATTRIBUTES,(void *)&num))) {
@@ -579,7 +590,7 @@ static GstBMLV *gst_bml_add_voice(GstBML *bml,GType voice_type) {
   GstBMLV *bmlv;
   gchar *name;
 
-  GST_DEBUG("adding a new voice to %p, current nr of voices is %lu",bml->self,bml->num_voices);
+  GST_DEBUG_OBJECT(bml->self,"adding a new voice to %p, current nr of voices is %lu",bml->self,bml->num_voices);
 
   bmlv=g_object_new(voice_type,NULL);
   //bmlv->parent=bml;
@@ -596,7 +607,7 @@ static GstBMLV *gst_bml_add_voice(GstBML *bml,GType voice_type) {
   bml->voices=g_list_append(bml->voices,bmlv);
   bml->num_voices++;
 
-  GST_DEBUG("added a new voice");
+  GST_DEBUG_OBJECT(bml->self,"added a new voice");
   return(bmlv);
 }
 
@@ -610,11 +621,11 @@ static void gst_bml_del_voice(GstBML *bml,GType voice_type) {
   GList *node;
   GstObject *obj;
 
-  GST_DEBUG("removing last voice to %p, current nr of voices is %lu",bml->self,bml->num_voices);
+  GST_DEBUG_OBJECT(bml->self,"removing last voice to %p, current nr of voices is %lu",bml->self,bml->num_voices);
 
   node=g_list_last(bml->voices);
   obj=node->data;
-  GST_DEBUG("  free voice : %p (%d)",obj,G_OBJECT(obj)->ref_count);
+  GST_DEBUG_OBJECT(bml->self,"  free voice : %p (%d)",obj,G_OBJECT(obj)->ref_count);
   gst_object_unparent(obj);
   // no need to unref, the unparent does that
   //g_object_unref(node->data);
@@ -622,7 +633,7 @@ static void gst_bml_del_voice(GstBML *bml,GType voice_type) {
   bml->voices=g_list_delete_link(bml->voices,node);
   bml->num_voices--;
 
-  GST_DEBUG("removed last voice");
+  GST_DEBUG_OBJECT(bml->self,"removed last voice");
 }
 
 /*
@@ -633,24 +644,24 @@ static void gst_bml_del_voice(GstBML *bml,GType voice_type) {
 static void gst_bml_init_voices(GstBML *bml,GstBMLClass *klass) {
   gpointer bmh=klass->bmh;
 
-  GST_INFO("initializing voices: bml=%p, bml_class=%p",bml,klass);
+  GST_INFO_OBJECT(bml->self,"initializing voices: bml=%p, bml_class=%p",bml,klass);
 
   bml->num_voices=0;
   bml->voices=NULL;
   if(bml(gstbml_is_polyphonic(bmh))) {
     gint i,min_voices;
 
-    GST_DEBUG("instantiating default voices");
+    GST_DEBUG_OBJECT(bml->self,"instantiating default voices");
 
     // add voice instances
     if(bml(get_machine_info(bmh,BM_PROP_MIN_TRACKS,(void *)&min_voices))) {
-      GST_DEBUG("adding %d voices",min_voices);
+      GST_DEBUG_OBJECT(bml->self,"adding %d voices",min_voices);
       for(i=0;i<min_voices;i++) {
         gst_bml_add_voice(bml,klass->voice_type);
       }
     }
     else {
-      GST_WARNING("failed to get min voices");
+      GST_WARNING_OBJECT(bml->self,"failed to get min voices");
     }
   }
 }
@@ -661,7 +672,7 @@ static void gst_bml_init_voices(GstBML *bml,GstBMLClass *klass) {
  * initialize the new instance
  */
 void bml(gstbml_init(GstBML *bml,GstBMLClass *klass,GstElement *element)) {
-  GST_DEBUG("init: element=%p, bml=%p, bml_class=%p",element,bml,klass);
+  GST_DEBUG_OBJECT(element,"init: element=%p, bml=%p, bml_class=%p",element,bml,klass);
 
   bml->self=element;
 
@@ -674,7 +685,7 @@ void bml(gstbml_init(GstBML *bml,GstBMLClass *klass,GstElement *element)) {
   // allocate the various arrays
   bml->srcpads=g_new0(GstPad *,klass->numsrcpads);
   bml->sinkpads=g_new0(GstPad *,klass->numsinkpads);
-  
+
   bml->triggers_changed=g_new0(gint,klass->numglobalparams+klass->numtrackparams);
 
   // nonzero default needed to instantiate() some plugins
@@ -684,7 +695,7 @@ void bml(gstbml_init(GstBML *bml,GstBMLClass *klass,GstElement *element)) {
   bml->subticks_per_tick=1;
   gstbml_calculate_buffer_frames(bml);
   bml(set_master_info(bml->beats_per_minute,bml->ticks_per_beat,bml->samplerate));
-  GST_DEBUG("activating %lu voice(s)",bml->num_voices);
+  GST_DEBUG_OBJECT(element,"activating %lu voice(s)",bml->num_voices);
   //bml(set_num_tracks(bml,bml->num_voices));
 }
 
@@ -707,31 +718,32 @@ void bml(gstbml_init_pads(GstElement *element, GstBML *bml, GstPadLinkFunction *
 
     if(GST_PAD_DIRECTION(pad)==GST_PAD_SINK) {
       bml->sinkpads[sinkcount++]=pad;
-      GST_INFO("  added sinkpad");
+      GST_INFO_OBJECT(element,"  added sinkpad");
     }
     else {
       bml->srcpads[srccount++]=pad;
-      GST_INFO("  added srcpad");
+      GST_INFO_OBJECT(element,"  added srcpad");
     }
   }
-  GST_INFO("  src_ct=%d, sink_ct=%d",srccount,sinkcount);
+  GST_INFO_OBJECT(element,"  src_ct=%d, sink_ct=%d",srccount,sinkcount);
 }
 
 void bml(gstbml_finalize(GstBML *bml)) {
-  GST_DEBUG("!!!! bml=%p",bml);
+  GST_DEBUG_OBJECT(bml->self,"!!!! bml=%p",bml);
 
   // free list of voices
   if(bml->voices) {
     g_list_free(bml->voices);
     bml->voices=NULL;
   }
-  
+
   g_free(bml->srcpads);
   g_free(bml->sinkpads);
-  
+
   g_free(bml->triggers_changed);
 
   bml(free(bml->bm));
+  bml->bm=NULL;
 }
 
 void bml(gstbml_set_property(GstBML *bml, GstBMLClass *bml_class, guint prop_id, const GValue *value, GParamSpec * pspec)) {
@@ -739,29 +751,29 @@ void bml(gstbml_set_property(GstBML *bml, GstBMLClass *bml_class, guint prop_id,
   gpointer bm=bml->bm;
   guint props_skip=ARG_LAST-1;
 
-  GST_DEBUG ("prop-id %d", prop_id);
+  GST_DEBUG_OBJECT(bml->self,"prop-id %d",prop_id);
 
   switch(prop_id) {
     // handle properties <ARG_LAST first
     case ARG_BPM:
     case ARG_TPB:
     case ARG_STPT:
-      GST_WARNING("use gst_bml_tempo_change_tempo()");
+      GST_WARNING_OBJECT(bml->self,"use gst_bml_tempo_change_tempo()");
       handled=TRUE;
       break;
     case ARG_HOST_CALLBACKS:
       // supply the callbacks to bml
-      GST_DEBUG("passing callbacks to bml");
+      GST_DEBUG_OBJECT(bml->self,"passing callbacks to bml");
       bml(set_callbacks(bm,g_value_get_pointer(value)));
       handled=TRUE;
-      break;      
+      break;
     default:
       if(bml(gstbml_is_polyphonic(bm))) {
         if(prop_id==(props_skip+1)) {
           gulong i;
           gulong old_num_voices=bml->num_voices;
           gulong new_num_voices = g_value_get_ulong(value);
-          GST_DEBUG ("change number of voices from %lu to %lu", old_num_voices,new_num_voices);
+          GST_DEBUG_OBJECT(bml->self,"change number of voices from %lu to %lu", old_num_voices,new_num_voices);
           // add or free voices
           if(old_num_voices<new_num_voices) {
             for(i=old_num_voices;i<new_num_voices;i++) {
@@ -782,13 +794,15 @@ void bml(gstbml_set_property(GstBML *bml, GstBMLClass *bml_class, guint prop_id,
         }
         props_skip++;
       }
+#if !GST_CHECK_VERSION(0,10,31)
       if(bml_class->help_uri) {
         if(prop_id==(props_skip+1)) {
-          GST_WARNING ("documentation-uri property is read only");
+          GST_WARNING_OBJECT(bml->self,"documentation-uri property is read only");
           handled=TRUE;
         }
         props_skip++;
       }
+#endif
       break;
   }
   prop_id-=props_skip;
@@ -806,7 +820,7 @@ void bml(gstbml_set_property(GstBML *bml, GstBMLClass *bml_class, guint prop_id,
     type=GPOINTER_TO_INT(g_param_spec_get_qdata(pspec,gst_bml_property_meta_quark_type));
 
     prop_id--;
-    GST_LOG("id: %d, attr: %d, global: %d, voice: %d",prop_id,bml_class->numattributes,bml_class->numglobalparams,bml_class->numtrackparams);
+    GST_LOG_OBJECT(bml->self,"id: %d, attr: %d, global: %d, voice: %d",prop_id,bml_class->numattributes,bml_class->numglobalparams,bml_class->numtrackparams);
      // is it an attribute ?
     if(prop_id<bml_class->numattributes) {
       bml(set_attribute_value(bm,prop_id,g_value_get_int(value)));
@@ -853,23 +867,23 @@ void bml(gstbml_get_property(GstBML *bml, GstBMLClass *bml_class, guint prop_id,
   gpointer bm=bml->bm;
   guint props_skip=ARG_LAST-1;
 
-  GST_DEBUG ("prop-id %d", prop_id);
+  GST_DEBUG_OBJECT(bml->self,"prop-id %d",prop_id);
 
   switch(prop_id) {
     // handle properties <ARG_LAST first
     case ARG_BPM:
       g_value_set_ulong(value, bml->beats_per_minute);
-      GST_DEBUG("requested BPM = %lu", bml->beats_per_minute);
+      GST_DEBUG_OBJECT(bml->self,"requested BPM = %lu", bml->beats_per_minute);
       handled=TRUE;
       break;
     case ARG_TPB:
       g_value_set_ulong(value, bml->ticks_per_beat);
-      GST_DEBUG("requested TPB = %lu", bml->ticks_per_beat);
+      GST_DEBUG_OBJECT(bml->self,"requested TPB = %lu", bml->ticks_per_beat);
       handled=TRUE;
       break;
     case ARG_STPT:
       g_value_set_ulong(value, bml->subticks_per_tick);
-      GST_DEBUG("requested STPB = %lu", bml->subticks_per_tick);
+      GST_DEBUG_OBJECT(bml->self,"requested STPB = %lu", bml->subticks_per_tick);
       handled=TRUE;
       break;
     /*case ARG_HOST_CALLBACKS:
@@ -880,11 +894,12 @@ void bml(gstbml_get_property(GstBML *bml, GstBMLClass *bml_class, guint prop_id,
       if(bml(gstbml_is_polyphonic(bm))) {
         if(prop_id==(props_skip+1)) {
           g_value_set_ulong(value, bml->num_voices);
-          GST_DEBUG("requested number of voices = %lu", bml->num_voices);
+          GST_DEBUG_OBJECT(bml->self,"requested number of voices = %lu", bml->num_voices);
           handled=TRUE;
         }
         props_skip++;
       }
+#if !GST_CHECK_VERSION(0,10,31)
       if(bml_class->help_uri) {
         if(prop_id==(props_skip+1)) {
           g_value_set_string(value,bml_class->help_uri);
@@ -892,6 +907,7 @@ void bml(gstbml_get_property(GstBML *bml, GstBMLClass *bml_class, guint prop_id,
         }
         props_skip++;
       }
+#endif
       break;
   }
   prop_id-=props_skip;
@@ -908,7 +924,7 @@ void bml(gstbml_get_property(GstBML *bml, GstBMLClass *bml_class, guint prop_id,
 
     type=GPOINTER_TO_INT(g_param_spec_get_qdata(pspec,gst_bml_property_meta_quark_type));
     prop_id--;
-    GST_DEBUG("id: %d, attr: %d, global: %d, voice: %d",prop_id,bml_class->numattributes,bml_class->numglobalparams,bml_class->numtrackparams);
+    GST_DEBUG_OBJECT(bml->self,"id: %d, attr: %d, global: %d, voice: %d",prop_id,bml_class->numattributes,bml_class->numglobalparams,bml_class->numtrackparams);
      // is it an attribute ?
     if(prop_id<bml_class->numattributes) {
       g_value_set_int(value,bml(get_attribute_value(bm,prop_id)));
@@ -970,7 +986,7 @@ void bml(gstbml_sync_values(GstBML *bml, GstBMLClass *bml_class)) {
   //if(G_UNLIKELY(!res)) { GST_WARNING("global sync failed"); }
   for(node=bml->voices;node;node=g_list_next(node)) {
     bmlv=node->data;
-    bmlv_class=GST_BMLV_GET_CLASS(bmlv);  
+    bmlv_class=GST_BMLV_GET_CLASS(bmlv);
     for(i=0;i<bmlv_class->numtrackparams;i++) {
       g_atomic_int_compare_and_exchange(&bmlv->triggers_changed[i],1,2);
     }
@@ -1015,7 +1031,7 @@ void bml(gstbml_reset_triggers(GstBML *bml, GstBMLClass *bml_class)) {
   GParamSpec *pspec;
   gulong i,v;
   gpointer addr;
-  
+
   for(i=0;i<bml_class->numglobalparams;i++) {
     if(g_atomic_int_compare_and_exchange(&bml->triggers_changed[i],2,0)) {
       pspec=bml_class->global_property[i];
@@ -1032,7 +1048,7 @@ void bml(gstbml_reset_triggers(GstBML *bml, GstBMLClass *bml_class)) {
   }
   for(v=0,node=bml->voices;node;node=g_list_next(node),v++) {
     bmlv=node->data;
-    bmlv_class=GST_BMLV_GET_CLASS(bmlv);  
+    bmlv_class=GST_BMLV_GET_CLASS(bmlv);
     for(i=0;i<bmlv_class->numtrackparams;i++) {
       if(g_atomic_int_compare_and_exchange(&bmlv->triggers_changed[i],2,0)) {
         pspec=bmlv_class->track_property[i];
